// Leadwerks "taper off" SSAO shader
// Mostly rewritten by Ingemar (AO2, new readDepth)

#version 150

uniform sampler2D texture0; // Z-buffer
uniform sampler2D texture1; // Original image

//uniform vec2 camerarange;
//uniform vec2 screensize;

vec2 camerarange = vec2(100.0, 1.0);
vec2 screensize = vec2(512.0, 512.0);

in vec2 texCoord;
out vec4 outColor;

float readDepth( in vec2 coord )
{
	float near = 1.0;
	float far = 100.0;
	float z = 2.0 * far * near / ((far - near)*texture(texture0, coord).x - far - near);
//	return z;

	// These numbers must match the main program's perspective projection
	float zNear = 1.0;
	float zFar = 100.0;

    float z_from_depth_texture = texture(texture0, coord).x;
    float z_sb = 2.0 * z_from_depth_texture - 1.0; // scale and bias from texture to normalized coords
    float z_world = 2.0 * zNear * zFar / (zFar + zNear - z_sb * (zFar - zNear)); // Get back to real Z
	return z_world;

// Undocumented garbage	
//	return (2.0 * camerarange.x) / (camerarange.y + camerarange.x - texture( texture0, coord ).x * (camerarange.y - camerarange.x));
}

float compareDepths( in float depth1, in float depth2 )
{
	float aoCap = 1.0;
	float aoMultiplier=10000.0;
	float depthTolerance=0.000;
	float aorange = 10.0;// units in space the AO effect extends to (this gets divided by the camera far range
	float diff = sqrt( clamp(1.0-(depth1-depth2) / (aorange/(camerarange.y-camerarange.x)),0.0,1.0) );
	float ao = min(aoCap,max(0.0,depth1-depth2-depthTolerance) * aoMultiplier) * diff;
	return ao;
}

float myCompareDepths( in float d, in float d1, in float d2 )
{
//	const float a = 5.0; // Difference that gives max occlusion
	const float a = 1.0; // Difference that gives max occlusion
	
	float dav = (d1+d2)/2.0;
	float x = d - dav;
//	float x = dav - d;
	float dao = max( 1.0/a/a*x*(2.0*a - x) - 0.05, 0.0); // negative - no occlusion
	return dao;
}

void main(void)
{	
	float depth = readDepth( texCoord );
	
	// depth = center pixel depth
	
	float d, d1, d2;
	
	float pw = 1.0 / screensize.x;
	float ph = 1.0 / screensize.y;

	float aoCap = 1.0;

// Two different calculations, 4 directions and 8 directions
	float ao = 0.0;
	float ao2 = 0.0;
	
	float aoMultiplier=10000.0;

	float depthTolerance = 0.001;
	
	float aoscale=1.0;

// Falloff for my own SSAO
	float ao2Mod=3.0; // 1.0
	float ao2dampening = 0.5; // 0.5

	d1=readDepth( vec2(texCoord.x+pw,texCoord.y+ph)); // Diagonal
	d2=readDepth( vec2(texCoord.x-pw,texCoord.y-ph));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);

	d1=readDepth( vec2(texCoord.x+pw,texCoord.y-ph)); // Diagonal
	d2=readDepth( vec2(texCoord.x-pw,texCoord.y+ph));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);

	d1=readDepth( vec2(texCoord.x+pw,texCoord.y)); // Along x
	d2=readDepth( vec2(texCoord.x-pw,texCoord.y));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);

	d1=readDepth( vec2(texCoord.x,texCoord.y+ph)); // Along y
	d2=readDepth( vec2(texCoord.x,texCoord.y-ph));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);


	d=readDepth( vec2(texCoord.x+pw,texCoord.y+ph));
	ao+=compareDepths(depth,d)/aoscale;

	d=readDepth( vec2(texCoord.x-pw,texCoord.y+ph));
	ao+=compareDepths(depth,d)/aoscale;

	d=readDepth( vec2(texCoord.x+pw,texCoord.y-ph));
	ao+=compareDepths(depth,d)/aoscale;

	d=readDepth( vec2(texCoord.x-pw,texCoord.y-ph));
	ao+=compareDepths(depth,d)/aoscale;
	
	pw*=2.0;
	ph*=2.0;
	aoMultiplier/=2.0;
	aoscale*=1.2;
	ao2Mod *= ao2dampening;


	d1=readDepth( vec2(texCoord.x+pw,texCoord.y+ph)); // Diagonal
	d2=readDepth( vec2(texCoord.x-pw,texCoord.y-ph));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);

	d1=readDepth( vec2(texCoord.x+pw,texCoord.y-ph)); // Diagonal
	d2=readDepth( vec2(texCoord.x-pw,texCoord.y+ph));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);

	d1=readDepth( vec2(texCoord.x+pw,texCoord.y)); // Along x
	d2=readDepth( vec2(texCoord.x-pw,texCoord.y));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);

	d1=readDepth( vec2(texCoord.x,texCoord.y+ph)); // Along y
	d2=readDepth( vec2(texCoord.x,texCoord.y-ph));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);


	d=readDepth( vec2(texCoord.x+pw,texCoord.y+ph));
	ao+=compareDepths(depth,d)/aoscale;

	d=readDepth( vec2(texCoord.x-pw,texCoord.y+ph));
	ao+=compareDepths(depth,d)/aoscale;

	d=readDepth( vec2(texCoord.x+pw,texCoord.y-ph));
	ao+=compareDepths(depth,d)/aoscale;

	d=readDepth( vec2(texCoord.x-pw,texCoord.y-ph));
	ao+=compareDepths(depth,d)/aoscale;

	pw*=2.0;
	ph*=2.0;
	aoMultiplier/=2.0;
	aoscale*=1.2;
	ao2Mod *= ao2dampening;

	d1=readDepth( vec2(texCoord.x+pw,texCoord.y+ph)); // Diagonal
	d2=readDepth( vec2(texCoord.x-pw,texCoord.y-ph));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);

	d1=readDepth( vec2(texCoord.x+pw,texCoord.y-ph)); // Diagonal
	d2=readDepth( vec2(texCoord.x-pw,texCoord.y+ph));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);

	d1=readDepth( vec2(texCoord.x+pw,texCoord.y)); // Along x
	d2=readDepth( vec2(texCoord.x-pw,texCoord.y));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);

	d1=readDepth( vec2(texCoord.x,texCoord.y+ph)); // Along y
	d2=readDepth( vec2(texCoord.x,texCoord.y-ph));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);

	d=readDepth( vec2(texCoord.x+pw,texCoord.y+ph));
	ao+=compareDepths(depth,d)/aoscale;

	d=readDepth( vec2(texCoord.x-pw,texCoord.y+ph));
	ao+=compareDepths(depth,d)/aoscale;

	d=readDepth( vec2(texCoord.x+pw,texCoord.y-ph));
	ao+=compareDepths(depth,d)/aoscale;

	d=readDepth( vec2(texCoord.x-pw,texCoord.y-ph));
	ao+=compareDepths(depth,d)/aoscale;
	
	pw*=2.0;
	ph*=2.0;
	aoMultiplier/=2.0;
	aoscale*=1.2;
	ao2Mod *= ao2dampening;
	
	
	d1=readDepth( vec2(texCoord.x+pw,texCoord.y+ph)); // Diagonal
	d2=readDepth( vec2(texCoord.x-pw,texCoord.y-ph));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);
	
	d1=readDepth( vec2(texCoord.x+pw,texCoord.y-ph)); // Diagonal
	d2=readDepth( vec2(texCoord.x-pw,texCoord.y+ph));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);
	
	d1=readDepth( vec2(texCoord.x+pw,texCoord.y)); // Along x
	d2=readDepth( vec2(texCoord.x-pw,texCoord.y));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);
	
	d1=readDepth( vec2(texCoord.x,texCoord.y+ph)); // Along y
	d2=readDepth( vec2(texCoord.x,texCoord.y-ph));
	ao2 += ao2Mod*myCompareDepths(depth, d1, d2);
	
	
	d=readDepth( vec2(texCoord.x+pw,texCoord.y+ph));
	ao+=compareDepths(depth,d)/aoscale;
	
	d=readDepth( vec2(texCoord.x-pw,texCoord.y+ph));
	ao+=compareDepths(depth,d)/aoscale;
	
	d=readDepth( vec2(texCoord.x+pw,texCoord.y-ph));
	ao+=compareDepths(depth,d)/aoscale;
	
	d=readDepth( vec2(texCoord.x-pw,texCoord.y-ph));
	ao+=compareDepths(depth,d)/aoscale;
	
	ao/=16.0;
//	ao = max(0.40, ao); // Cut away garbage in low areas (ugly)
	ao2/=2.0;
//	ao2 = sqrt(ao2);
	
	
//	outColor = vec4(1.0-ao) * texture(texture1,texCoord);
//	outColor = vec4(1.0-ao); // AO values only
	outColor = vec4(depth / 50.0); // linear depth values
//	outColor = vec4(1.0-ao2); // AO values only
	outColor = vec4(1.0-ao2) * texture(texture1,texCoord); // AO values + scene
//	outColor = texture(texture1,texCoord); // Image
//	outColor = texture(texture0,texCoord); // Depth

//	outColor = vec4(1.4-ao); // AO values only (+0.4 to show AO only)
}
